﻿using Microsoft.VisualBasic.FileIO;
using ProjectAtelier.Entities;
using ProjectAtelier.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAtelier.Forms
{
    public partial class FormClient : Form
    {
        private readonly IClientRepository _clientRepository;
        private int? _clientId;

        public int Id
        {
            set
            {
                try
                {
                    var client = _clientRepository.ReadClientById(value);
                    if (client == null)
                    {
                        throw new InvalidDataException(nameof(client));
                    }
                    
                    foreach (Gender elem in Enum.GetValues(typeof(Gender)))
                    {
                        if ((elem & client.Gender) != 0)
                        {
                            checkedListBoxGender.SetItemChecked(checkedListBoxGender.Items.IndexOf(
                            elem), true);
                        }
                    }
                    textBoxName.Text = client.Name;
                    textBoxPhone.Text = client.Phone;
                    _clientId = value;
                

                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при получении данных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }

        public FormClient(IClientRepository clientRepository)
        {
            InitializeComponent();
            _clientRepository = clientRepository ??
            throw new ArgumentNullException(nameof(clientRepository));
            foreach (var elem in Enum.GetValues(typeof(Gender)))
            {
                checkedListBoxGender.Items.Add(elem);
            }

        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxName.Text) ||
                string.IsNullOrWhiteSpace(textBoxPhone.Text) ||
                checkedListBoxGender.CheckedItems.Count == 0)
                {
                    throw new Exception("Имеются незаполненные поля");
                }
                if (_clientId.HasValue)
                {
                    _clientRepository.UpdateClient(CreateClient(_clientId.Value));
                }
                else
                {
                    _clientRepository.CreateClient(CreateClient(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void ButtonCancel_Click(object sender, EventArgs e) =>Close();
        private Client CreateClient(int id)
        {
            Gender gender = Gender.None;
            foreach (var elem in checkedListBoxGender.CheckedItems)
            {
                gender |= (Gender)elem;
            }
            return Client.CreateEntity(id, textBoxName.Text, textBoxPhone.Text, gender);
        }
    }

}