﻿using ProjectAtelier.Entities;
using ProjectAtelier.Repositories;
using ProjectAtelier.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAtelier.Forms
{
    public partial class FormProductMaterial : Form
    {
        private readonly IProductRepository _productRepository;
        private readonly IMaterialRepository _materialRepository;
        private int? _productId;
        public int Id
        {
            set
            {
                try
                {
                    var product = _productRepository.ReadProductById(value);
                    if (product == null)
                    {
                        throw new InvalidDataException(nameof(product));
                    }
                    textBoxName.Text = product.Name;
                    comboBoxProduct.SelectedItem = product.View;
                    numericUpDownCount.Value = product.CountMaterial;
                    _productId = value;

                    dataGridView.Rows.Clear();
                    var rows = new List<DataGridViewRow>();
                    foreach (var productMaterial in product.ProductMaterial)
                    {
                        var material = _materialRepository.ReadMaterialById(productMaterial.MaterialId);
                        if (material != null)
                        {
                            var row = new DataGridViewRow();
                            row.CreateCells(dataGridView, material.Id, productMaterial.Count);
                            rows.Add(row);
                        }
                    }
                    dataGridView.Rows.AddRange(rows.ToArray());
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при полученииданных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }
        public FormProductMaterial(IProductRepository productRepository, IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
            _materialRepository = materialRepository ?? throw new ArgumentNullException(nameof(materialRepository));
            comboBoxProduct.DataSource = Enum.GetValues(typeof(ProductView));

            ColumnMaterial.DataSource = materialRepository.ReadMaterials();
            ColumnMaterial.DisplayMember = "Name";
            ColumnMaterial.ValueMember = "Id";
        }
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (dataGridView.RowCount < 1)
            {
                throw new Exception("Имеются незаполненны поля");
            }
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxName.Text) || comboBoxProduct.SelectedIndex < 1)
                {
                    throw new Exception("Имеются незаполненные поля");
                }
                if (_productId.HasValue)
                {
                    _productRepository.UpdateProduct(CreateProduct(_productId.Value));
                }
                else
                {
                    _productRepository.CreateProduct(CreateProduct(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void buttonCancel_Click(object sender, EventArgs e) => Close();
        private List<ProductMaterial> CreateListMaterialFromDataGrid()
        {
            var list = new List<ProductMaterial>();
            foreach (DataGridViewRow row in dataGridView.Rows)
            {
                if (row.Cells["ColumnMaterial"].Value == null || row.Cells["ColumnCount"].Value == null)
                {
                    continue;
                }
                list.Add(ProductMaterial.CreateOperation(0, Convert.ToInt32(row.Cells["ColumnMaterial"].Value), Convert.ToInt32(row.Cells["ColumnCount"].Value)));
            }
            return list;
        }
        private Product CreateProduct(int id) => Product.CreateEntity(id, textBoxName.Text, (ProductView)comboBoxProduct.SelectedItem!, Convert.ToInt32(numericUpDownCount.Value), CreateListMaterialFromDataGrid());

    }
}