﻿using ProjectAtelier.Entities;
using ProjectAtelier.Repositories;
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;

namespace ProjectAtelier.Forms
{
    public partial class FormProductOrder : Form
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IProductRepository _productRepository;
        private readonly IClientRepository _clientRepository; // Добавляем репозиторий для клиентов

        public FormProductOrder(IOrderRepository orderRepository, IProductRepository productRepository, IClientRepository clientRepository)
        {
            InitializeComponent();
            _orderRepository = orderRepository ?? throw new ArgumentNullException(nameof(orderRepository));
            _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
            _clientRepository = clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));

            comboBoxStatus.DataSource = Enum.GetValues(typeof(OrderStatus));
            // Заполняем ComboBox для выбора клиента
            comboBoxClient.DataSource = clientRepository.ReadClients();
            comboBoxClient.DisplayMember = "Name";
            comboBoxClient.ValueMember = "Id";

            ColumProduct.DataSource = productRepository.ReadProducts();
            ColumProduct.DisplayMember = "FullProductName";
            ColumProduct.ValueMember = "Id";

            // Инициализация DateTimePicker
            dateTimePicker.Format = DateTimePickerFormat.Custom;
            dateTimePicker.CustomFormat = "yyyy-MM-dd ";
            dateTimePicker.Value = DateTime.Now; // Устанавливаем текущую дату и время по умолчанию
            

        }

        private void ButtonAdd_Click(object sender, EventArgs e)
        {
            if (dataGridView.RowCount < 1 || textBoxCharacteristic.Text == null || comboBoxStatus.SelectedIndex < 0 || comboBoxClient.SelectedIndex < 0)
            {
                throw new Exception("Имеются незаполненные поля");
            }
            try
            {
                int clientId = (int)comboBoxClient.SelectedValue;
                DateTime selectedDateTime = dateTimePicker.Value; // Получаем выбранную дату и время
                _orderRepository.CreateOrder(Order.CreateOperation(0, (OrderStatus)comboBoxStatus.SelectedValue!, textBoxCharacteristic.Text, CreateListProductFromDataGrid(), clientId, selectedDateTime));
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e) => Close();
        private List<OrderProduct> CreateListProductFromDataGrid()
        {
            var list = new List<OrderProduct>();
            foreach (DataGridViewRow row in dataGridView.Rows)
            {
                if (row.Cells["ColumProduct"].Value == null ||
                row.Cells["ColumnCount"].Value == null)
                {
                    continue;
                }
                list.Add(OrderProduct.CreateOperation(0,Convert.ToInt32(row.Cells["ColumProduct"].Value), Convert.ToInt32(row.Cells["ColumnCount"].Value)));
            }
            return list;
        }
       
    }
}