using ProjectAtelier.Repositories.Implementations;
using ProjectAtelier.Repositories;
using ProjectAtelier.REPOSITORY.Implementations;
using ProjectAtelier.REPOSITORY;
using Unity;
using Unity.Microsoft.Logging;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;

namespace ProjectAtelier
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormAtelier>());
        }
        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer();
            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<IOrderRepository, OrderRepository>();
            container.RegisterType<IMaterialRepository, MaterialRepository>();
            container.RegisterType<IProductRepository, ProductRepository>();
            container.RegisterType<IClientRepository, ClientRepository>();
            container.RegisterType<IProductMaterialRepository, ProductMaterialRepository>();
            container.RegisterType<IMaterialReplenishmentRepository, MaterialReplenishmentRepository>();
            container.RegisterType<IOrderMaterialsRepository, OrderMaterialsRepository>();

            container.RegisterType<IConnectionString, ConnectionString>();
            return container;
        }
        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json")
                .Build())
                .CreateLogger());
            return loggerFactory;
        }
    }
}